/*
**  NoteInfoWindowController.m
**
**  Copyright (c) 2001
**
**  Author: Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#import "NoteInfoWindowController.h"

#import "Constants.h"
#import "Note.h"

#ifndef MACOSX
#import "NoteInfoWindow.h"
#endif

static NoteInfoWindowController *singleInstance;

@implementation NoteInfoWindowController

- (id) initWithWindowNibName: (NSString *) theNibName
{
#ifdef MACOSX
  self = [super initWithWindowNibName: theNibName];
#else
  NoteInfoWindow *noteInfoWindow;
  
  noteInfoWindow = [[NoteInfoWindow alloc] initWithContentRect:NSMakeRect(100,100,320,110)
                                      styleMask: (NSClosableWindowMask|NSResizableWindowMask)
                                      backing: NSBackingStoreBuffered
                                      defer: NO];

  self = [super initWithWindow: noteInfoWindow];
  
  [noteInfoWindow layoutWindow];
  [noteInfoWindow setDelegate: self];

  // We link our outlets
  creationDateField = [noteInfoWindow creationDateField];
  modificationDateField = [noteInfoWindow modificationDateField];

#endif
 
    // We set the title to an empty string
    [[self window] setTitle: _(@"Note Information")];
    
    return self;
}

- (void) dealloc
{
  singleInstance = nil;
  
  [super dealloc];
}

//
// delegate methods
//

- (void) windowDidLoad
{
  [super windowDidLoad];
}

- (void) windowWillClose: (NSNotification *) theNotification
{
  AUTORELEASE(self);
}


//
// access / mutation
//


- (void) setNote: (Note *) theNote
{
  if (! singleInstance )
    {
      return;
    }

  if ( theNote )
    {
      note = theNote;
      
      [creationDateField setStringValue: [[note creationDate] description] ];
      [modificationDateField setStringValue: [[note modificationDate] description] ];
    }
  else
    {
      note = nil;
    }
}

//
// single instance
//

+ (id) singleInstance
{
  if (!singleInstance)
    {
      singleInstance = [[NoteInfoWindowController alloc] initWithWindowNibName: @"NoteInfoWindow"];
    }
    
  return singleInstance;
}

@end
